'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.stringifyValues = stringifyValues;
exports.normalizeNovelType = normalizeNovelType;
exports.createUri = createUri;
exports.fetchNodeJs = fetchNodeJs;
exports.fetchBrowser = fetchBrowser;
exports.fetch = fetch;

var _schema = require('./schema');

var _schema2 = _interopRequireDefault(_schema);

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _zlib = require('zlib');

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _fetchJsonp = require('fetch-jsonp');

var _fetchJsonp2 = _interopRequireDefault(_fetchJsonp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var apis = {
  default: 'http://api.syosetu.com/novelapi/api/',
  r18: 'http://api.syosetu.com/novel18api/api/'
};

function stringifyValues() {
  var object = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var stringifidObject = {};

  Object.keys(object).forEach(function (key) {
    var value = object[key];

    switch (Object.prototype.toString.call(value)) {
      case '[object Number]':
        stringifidObject[key] = String(value);
        break;

      case '[object Array]':
        stringifidObject[key] = value.join('-');
        break;

      default:
        stringifidObject[key] = value;
    }
  });

  return stringifidObject;
}

function normalizeNovelType(items) {
  items.forEach(function (item) {
    if (item.noveltype !== undefined) {
      item.novel_type = item.noveltype;
      delete item.noveltype;
    }
  });
}

function createUri(params) {
  var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var api = opts.r18 ? apis.r18 : apis.default;

  var _schema$validate = _schema2.default.validate(stringifyValues(params));

  var error = _schema$validate.error;
  var value = _schema$validate.value;

  if (error) {
    throw error;
  }
  var data = _extends({}, value);
  if (opts.jsonp) {
    data.out = 'jsonp';
  } else {
    data.out = 'json';
    data.gzip = 5;
  }
  return api + '?' + _querystring2.default.stringify(data);
}

function fetchNodeJs(uri) {
  var options = {
    responseType: 'arraybuffer'
  };
  return _axios2.default.get(uri, options).then(function (response) {
    return response.data;
  }).then(function (data) {
    return new _bluebird2.default(function (resolve, reject) {
      (0, _zlib.gunzip)(data, function (error, buffer) {
        if (error) {
          reject(error);
          return;
        }
        resolve(buffer);
      });
    });
  }).then(function (buffer) {
    return JSON.parse(buffer.toString());
  });
}
function fetchBrowser(uri) {
  return (0, _fetchJsonp2.default)(uri).then(function (response) {
    return response.json();
  });
}

function fetch() {
  var params = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var uri = void 0;
  var promise = void 0;
  if (typeof window === 'undefined') {
    uri = createUri(params, opts);
    promise = fetchNodeJs(uri);
  } else {
    uri = createUri(params, _extends({}, opts, { jsonp: true }));
    promise = fetchBrowser(uri);
  }

  return promise.then(function (_ref) {
    var _ref2 = _toArray(_ref);

    var result = _ref2[0];

    var items = _ref2.slice(1);

    normalizeNovelType(items);

    return { uri: uri, allcount: result.allcount, items: items };
  });
}
//# sourceMappingURL=utils.js.map